@@ -1,8 +1,8 @@
 
 /*
  * Ekiga -- A VoIP and Video-Conferencing application
- * Copyright (C) 2000-2007 Damien Sandras
-
+ * Copyright (C) 2000-2009 Damien Sandras <dsandras@seconix.com>
+ *
  * This program is free software; you can  redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or (at
@@ -28,14 +28,16 @@
  *                         form-dialog.cpp  -  description
  *                         ------------------------------------------
  *   begin                : written in 2007 by Julien Puydt
- *   copyright            : (c) 2007 by Julien Puydt
+ *   copyright            : (c) 2007-2008 by Julien Puydt
  *   description          : implementation of a gtk+ representation of forms
  *
  */
 
+
 #include <algorithm>
 #include <cstring>
 #include <iostream>
+#include <glib/gi18n.h>
 
 #include "toolbox.h"
 #include "form-dialog-gtk.h"
@@ -103,9 +105,9 @@ editable_set_choice_toggled_cb (GtkCellRendererToggle *cell,
 /** Called when a link in a Form is clicked.
  * Open the URI.
  *
- * @param: The URI to open. 
+ * @param: The URI to open.
  */
-static void 
+static void
 link_clicked_cb (GtkWidget *widget,
                  gpointer data);
 
@@ -186,8 +188,10 @@ public:
 
   BooleanSubmitter (const std::string _name,
 		    const std::string _description,
+		    bool _advanced,
 		    GtkWidget *_widget): name(_name),
 					 description(_description),
+					 advanced(_advanced),
 					 widget(_widget)
   { }
 
@@ -197,13 +201,15 @@ public:
   void submit (Ekiga::FormBuilder &builder)
   {
     builder.boolean (name, description,
-		     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)));
+		     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)),
+		     advanced);
   }
 
 private:
 
   const std::string name;
   const std::string description;
+  bool advanced;
   GtkWidget *widget;
 };
 
@@ -214,8 +220,10 @@ public:
 
   TextSubmitter (const std::string _name,
 		 const std::string _description,
+		 bool _advanced,
 		 GtkWidget *_widget): name(_name),
 				      description(_description),
+				      advanced(_advanced),
 				      widget(_widget)
   { }
 
@@ -225,13 +233,15 @@ public:
   void submit (Ekiga::FormBuilder &builder)
   {
     builder.text (name, description,
-		  gtk_entry_get_text (GTK_ENTRY (widget)));
+		  gtk_entry_get_text (GTK_ENTRY (widget)),
+		  advanced);
   }
 
 private:
 
   const std::string name;
   const std::string description;
+  bool advanced;
   GtkWidget *widget;
 };
 
@@ -242,8 +252,10 @@ public:
 
   PrivateTextSubmitter (const std::string _name,
 			const std::string _description,
+			bool _advanced,
 			GtkWidget *_widget): name(_name),
 					     description(_description),
+					     advanced(_advanced),
 					     widget(_widget)
   { }
 
@@ -253,13 +265,15 @@ public:
   void submit (Ekiga::FormBuilder &builder)
   {
     builder.private_text (name, description,
-			  gtk_entry_get_text (GTK_ENTRY (widget)));
+			  gtk_entry_get_text (GTK_ENTRY (widget)),
+			  advanced);
   }
 
 private:
 
   const std::string name;
   const std::string description;
+  bool advanced;
   GtkWidget *widget;
 };
 
@@ -270,8 +284,10 @@ public:
 
   MultiTextSubmitter (const std::string _name,
 		      const std::string _description,
+		      bool _advanced,
 		      GtkTextBuffer *_buffer): name(_name),
 					       description(_description),
+					       advanced(_advanced),
 					       buffer(_buffer)
   { }
 
@@ -287,13 +303,15 @@ public:
     gtk_text_buffer_get_end_iter (buffer, &end);
     builder.multi_text (name, description,
 			gtk_text_buffer_get_text (buffer,
-						  &start, &end, FALSE));
+						  &start, &end, FALSE),
+			advanced);
   }
 
 private:
 
   const std::string name;
   const std::string description;
+  bool advanced;
   GtkTextBuffer *buffer;
 };
 
@@ -305,9 +323,11 @@ public:
   SingleChoiceSubmitter (const std::string _name,
 			 const std::string _description,
 			 const std::map<std::string, std::string> _choices,
+			 bool _advanced,
 			 GtkWidget *_combo): name(_name),
 					     description(_description),
 					     choices(_choices),
+					     advanced(_advanced),
 					     combo(_combo)
   { }
 
@@ -332,7 +352,8 @@ public:
     gtk_tree_model_get (model, &iter, COLUMN_VALUE, &cvalue, -1);
 
     builder.single_choice (name, description,
-			   std::string (cvalue), choices);
+			   std::string (cvalue), choices,
+			   advanced);
 
     g_free (cvalue);
   }
@@ -342,6 +363,7 @@ private:
   const std::string name;
   const std::string description;
   const std::map<std::string, std::string> choices;
+  bool advanced;
   GtkWidget *combo;
 };
 
@@ -353,9 +375,10 @@ public:
   MultipleChoiceSubmitter (const std::string _name,
 			   const std::string _description,
 			   const std::map<std::string, std::string> _choices,
+			   bool _advanced,
 			   GtkWidget *_tree_view):
     name(_name), description(_description),
-    choices(_choices), tree_view (_tree_view)
+    choices(_choices), advanced(_advanced), tree_view (_tree_view)
   { }
 
   ~MultipleChoiceSubmitter ()
@@ -403,7 +426,7 @@ public:
       } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (model), &iter));
     }
 
-    builder.multiple_choice (name, description, values, choices);
+    builder.multiple_choice (name, description, values, choices, advanced);
   }
 
 private:
@@ -411,6 +434,7 @@ private:
   const std::string name;
   const std::string description;
   std::map<std::string, std::string> choices;
+  bool advanced;
   GtkWidget *tree_view;
 };
 
@@ -421,8 +445,9 @@ public:
 
   EditableSetSubmitter (const std::string _name,
 			const std::string _description,
+			bool _advanced,
 			GtkWidget *_tree_view):
-    name(_name), description(_description), tree_view(_tree_view)
+    name(_name), description(_description), advanced(_advanced), tree_view(_tree_view)
   { }
 
   ~EditableSetSubmitter ()
@@ -466,13 +491,14 @@ public:
       } while (gtk_tree_model_iter_next (GTK_TREE_MODEL (model), &iter));
     }
 
-    builder.editable_set (name, description, values, proposed_values);
+    builder.editable_set (name, description, values, proposed_values, advanced);
   }
 
 private:
 
   const std::string name;
   const std::string description;
+  bool advanced;
   GtkWidget *tree_view;
 };
 
@@ -587,7 +613,7 @@ multiple_choice_choice_toggled_cb (G_GNUC_UNUSED GtkCellRendererToggle *cell,
 }
 
 
-static void 
+static void
 link_clicked_cb (GtkWidget * /*widget*/,
                  gpointer data)
 {
@@ -601,6 +627,7 @@ FormDialog::FormDialog (Ekiga::FormRequest &_request,
   GtkWidget *vbox = NULL;
 
   rows = 0;
+  advanced_rows = 0;
 
   window = gtk_dialog_new_with_buttons (NULL, GTK_WINDOW (NULL),
                                         GTK_DIALOG_MODAL,
@@ -617,6 +644,7 @@ FormDialog::FormDialog (Ekiga::FormRequest &_request,
   vbox = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox), vbox, FALSE, FALSE, 0);
+  gtk_widget_show (vbox);
 
   preamble = gtk_vbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), preamble, FALSE, FALSE, 0);
@@ -626,6 +654,13 @@ FormDialog::FormDialog (Ekiga::FormRequest &_request,
   gtk_table_set_col_spacings (GTK_TABLE (fields), 2);
   gtk_box_pack_start (GTK_BOX (vbox), fields, FALSE, FALSE, 3);
 
+  advanced_fields = gtk_table_new (0, 2, FALSE);
+  gtk_table_set_row_spacings (GTK_TABLE (advanced_fields), 2);
+  gtk_table_set_col_spacings (GTK_TABLE (advanced_fields), 2);
+  expander = gtk_expander_new (_("Advanced"));
+  gtk_container_add (GTK_CONTAINER (expander), advanced_fields);
+  gtk_box_pack_start (GTK_BOX (vbox), expander, FALSE, FALSE, 3);
+
   labels_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
   options_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
@@ -647,7 +682,11 @@ FormDialog::~FormDialog ()
 void
 FormDialog::run ()
 {
-  gtk_widget_show_all (window);
+  gtk_widget_show_all (preamble);
+  gtk_widget_show_all (fields);
+  if (advanced_rows > 0)
+    gtk_widget_show_all (expander);
+  gtk_widget_show (window);
   switch (gtk_dialog_run (GTK_DIALOG (window))) {
 
   case GTK_RESPONSE_ACCEPT:
@@ -751,27 +790,35 @@ FormDialog::hidden (const std::string name,
 void
 FormDialog::boolean (const std::string name,
 		     const std::string description,
-		     bool value)
+		     bool value,
+		     bool advanced)
 {
   GtkWidget *widget = NULL;
   BooleanSubmitter *submitter = NULL;
 
-  rows++;
-  gtk_table_resize (GTK_TABLE (fields), rows, 2);
+  grow_fields (advanced);
 
   widget = gtk_check_button_new_with_label (description.c_str ());
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), value);
-  gtk_table_attach_defaults (GTK_TABLE (fields), widget,
-			     0, 2, rows -1, rows);
+  if (advanced) {
 
-  submitter = new BooleanSubmitter (name, description, widget);
+    gtk_table_attach_defaults (GTK_TABLE (advanced_fields), widget,
+			       0, 2, advanced_rows -1, advanced_rows);
+  } else {
+
+    gtk_table_attach_defaults (GTK_TABLE (fields), widget,
+			       0, 2, rows -1, rows);
+  }
+
+  submitter = new BooleanSubmitter (name, description, advanced, widget);
   submitters.push_back (submitter);
 }
 
 void
 FormDialog::text (const std::string name,
 		  const std::string description,
-		  const std::string value)
+		  const std::string value,
+		  bool advanced)
 {
   GtkWidget *label = NULL;
   GtkWidget *widget = NULL;
@@ -779,8 +826,7 @@ FormDialog::text (const std::string name,
 
   gchar *label_text = NULL;
 
-  rows++;
-  gtk_table_resize (GTK_TABLE (fields), rows, 2);
+  grow_fields (advanced);
 
   label = gtk_label_new (NULL);
   gtk_size_group_add_widget (labels_group, label);
@@ -795,18 +841,33 @@ FormDialog::text (const std::string name,
   gtk_size_group_add_widget (options_group, widget);
   gtk_entry_set_text (GTK_ENTRY (widget), value.c_str ());
 
-  gtk_table_attach (GTK_TABLE (fields), label,
-                    0, 1, rows - 1, rows,
-                    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-                    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-                    0, 0);
-  gtk_table_attach (GTK_TABLE (fields), widget,
-		    1, 2, rows - 1, rows,
-                    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-                    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-                    0, 0);
+  if (advanced) {
 
-  submitter = new TextSubmitter (name, description, widget);
+    gtk_table_attach (GTK_TABLE (advanced_fields), label,
+		      0, 1, advanced_rows - 1, advanced_rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0, 0);
+    gtk_table_attach (GTK_TABLE (advanced_fields), widget,
+		      1, 2, advanced_rows - 1, advanced_rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0, 0);
+  } else {
+
+    gtk_table_attach (GTK_TABLE (fields), label,
+		      0, 1, rows - 1, rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0, 0);
+    gtk_table_attach (GTK_TABLE (fields), widget,
+		      1, 2, rows - 1, rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0, 0);
+  }
+
+  submitter = new TextSubmitter (name, description, advanced, widget);
   submitters.push_back (submitter);
 }
 
@@ -814,7 +875,8 @@ FormDialog::text (const std::string name,
 void
 FormDialog::private_text (const std::string name,
 			  const std::string description,
-			  const std::string value)
+			  const std::string value,
+			  bool advanced)
 {
   GtkWidget *label = NULL;
   GtkWidget *widget = NULL;
@@ -822,8 +884,7 @@ FormDialog::private_text (const std::string name,
 
   gchar *label_text = NULL;
 
-  rows++;
-  gtk_table_resize (GTK_TABLE (fields), rows, 2);
+  grow_fields (advanced);
 
   label = gtk_label_new (NULL);
   gtk_size_group_add_widget (labels_group, label);
@@ -839,18 +900,33 @@ FormDialog::private_text (const std::string name,
   gtk_size_group_add_widget (options_group, widget);
   gtk_entry_set_text (GTK_ENTRY (widget), value.c_str ());
 
-  gtk_table_attach (GTK_TABLE (fields), label,
-                    0, 1, rows - 1, rows,
-                    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-                    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-                    0, 0);
-  gtk_table_attach (GTK_TABLE (fields), widget,
-		    1, 2, rows - 1, rows,
-                    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-                    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-                    0, 0);
+  if (advanced) {
 
-  submitter = new PrivateTextSubmitter (name, description, widget);
+    gtk_table_attach (GTK_TABLE (advanced_fields), label,
+		      0, 1, advanced_rows - 1, advanced_rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0, 0);
+    gtk_table_attach (GTK_TABLE (advanced_fields), widget,
+		      1, 2, advanced_rows - 1, advanced_rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0, 0);
+  } else {
+
+    gtk_table_attach (GTK_TABLE (fields), label,
+		      0, 1, rows - 1, rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0, 0);
+    gtk_table_attach (GTK_TABLE (fields), widget,
+		      1, 2, rows - 1, rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0, 0);
+  }
+
+  submitter = new PrivateTextSubmitter (name, description, advanced, widget);
   submitters.push_back (submitter);
 }
 
@@ -858,7 +934,8 @@ FormDialog::private_text (const std::string name,
 void
 FormDialog::multi_text (const std::string name,
 			const std::string description,
-			const std::string value)
+			const std::string value,
+			bool advanced)
 {
   GtkWidget *label = NULL;
   GtkWidget *scroller = NULL;
@@ -866,16 +943,22 @@ FormDialog::multi_text (const std::string name,
   GtkTextBuffer *buffer = NULL;
   MultiTextSubmitter *submitter = NULL;
 
-  rows++;
-  gtk_table_resize (GTK_TABLE (fields), rows, 2);
+  grow_fields (advanced);
 
   label = gtk_label_new_with_mnemonic (description.c_str ());
   gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
   gtk_label_set_line_wrap_mode (GTK_LABEL (label), PANGO_WRAP_WORD);
-  gtk_table_attach_defaults (GTK_TABLE (fields), label,
-			     0, 2, rows -1, rows);
-  rows++;
-  gtk_table_resize (GTK_TABLE (fields), rows, 2);
+  if (advanced) {
+
+    gtk_table_attach_defaults (GTK_TABLE (advanced_fields), label,
+			       0, 2, advanced_rows -1, advanced_rows);
+  } else {
+
+    gtk_table_attach_defaults (GTK_TABLE (fields), label,
+			       0, 2, rows -1, rows);
+  }
+
+  grow_fields (advanced);
 
   widget = gtk_text_view_new ();
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (widget));
@@ -885,10 +968,18 @@ FormDialog::multi_text (const std::string name,
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroller),
 				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_container_add (GTK_CONTAINER (scroller), widget);
-  gtk_table_attach_defaults (GTK_TABLE (fields), scroller,
-			     0, 2, rows -1, rows);
 
-  submitter = new MultiTextSubmitter (name, description, buffer);
+  if (advanced) {
+
+    gtk_table_attach_defaults (GTK_TABLE (advanced_fields), scroller,
+			       0, 2, advanced_rows -1, advanced_rows);
+  } else {
+
+    gtk_table_attach_defaults (GTK_TABLE (fields), scroller,
+			       0, 2, rows -1, rows);
+  }
+
+  submitter = new MultiTextSubmitter (name, description, advanced, buffer);
   submitters.push_back (submitter);
 }
 
@@ -897,7 +988,8 @@ void
 FormDialog::single_choice (const std::string name,
 			   const std::string description,
 			   const std::string value,
-			   const std::map<std::string, std::string> choices)
+			   const std::map<std::string, std::string> choices,
+			   bool advanced)
 {
   GtkWidget *label = NULL;
   gchar* label_text = NULL;
@@ -907,8 +999,7 @@ FormDialog::single_choice (const std::string name,
   GtkTreeIter iter;
   SingleChoiceSubmitter *submitter = NULL;
 
-  rows++;
-  gtk_table_resize (GTK_TABLE (fields), rows, 2);
+  grow_fields (advanced);
 
   label = gtk_label_new (NULL);
   gtk_size_group_add_widget (labels_group, label);
@@ -941,18 +1032,34 @@ FormDialog::single_choice (const std::string name,
       gtk_combo_box_set_active_iter (GTK_COMBO_BOX (widget), &iter);
   }
 
-  gtk_table_attach (GTK_TABLE (fields), label,
-		    0, 1, rows -1, rows,
-		    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-		    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-		    0,0);
-  gtk_table_attach (GTK_TABLE (fields), widget,
-		    1, 2, rows -1, rows,
-		    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-		    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-		    0,0);
+  if (advanced) {
 
-  submitter = new SingleChoiceSubmitter (name, description, choices, widget);
+    gtk_table_attach (GTK_TABLE (advanced_fields), label,
+		      0, 1, advanced_rows -1, advanced_rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0,0);
+    gtk_table_attach (GTK_TABLE (advanced_fields), widget,
+		      1, 2, advanced_rows -1, advanced_rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0,0);
+  } else {
+
+    gtk_table_attach (GTK_TABLE (fields), label,
+		      0, 1, rows -1, rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0,0);
+    gtk_table_attach (GTK_TABLE (fields), widget,
+		      1, 2, rows -1, rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0,0);
+  }
+
+  submitter = new SingleChoiceSubmitter (name, description, choices,
+					 advanced, widget);
   submitters.push_back (submitter);
 }
 
@@ -961,7 +1068,8 @@ void
 FormDialog::multiple_choice (const std::string name,
 			     const std::string description,
 			     const std::set<std::string> values,
-			     const std::map<std::string, std::string> choices)
+			     const std::map<std::string, std::string> choices,
+			     bool advanced)
 {
   GtkWidget *label = NULL;
   GtkWidget *scroll = NULL;
@@ -977,8 +1085,7 @@ FormDialog::multiple_choice (const std::string name,
 
   MultipleChoiceSubmitter *submitter = NULL;
 
-  rows++;
-  gtk_table_resize (GTK_TABLE (fields), rows, 2);
+  grow_fields (advanced);
 
   /* The label */
   label = gtk_label_new (NULL);
@@ -987,12 +1094,6 @@ FormDialog::multiple_choice (const std::string name,
   gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), label_text);
   g_free (label_text);
 
-  gtk_table_attach (GTK_TABLE (fields), label,
-                    0, 2, rows - 1, rows,
-                    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-                    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-                    0, 0);
-
   /* The GtkListStore containing the choices */
   tree_view = gtk_tree_view_new ();
   list_store = gtk_list_store_new (MultipleChoiceSubmitter::COLUMN_NUMBER,
@@ -1042,15 +1143,34 @@ FormDialog::multiple_choice (const std::string name,
                         -1);
   }
 
-  rows++;
-  gtk_table_resize (GTK_TABLE (fields), rows, 2);
-  gtk_table_attach (GTK_TABLE (fields), frame,
-                    0, 2, rows - 1, rows,
-                    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-                    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-                    0, 0);
+  if (advanced) {
 
-  submitter = new MultipleChoiceSubmitter (name, description, choices, tree_view);
+    gtk_table_attach (GTK_TABLE (advanced_fields), label,
+		      0, 2, advanced_rows - 1, advanced_rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0, 0);
+    gtk_table_attach (GTK_TABLE (advanced_fields), frame,
+		      0, 2, advanced_rows - 1, advanced_rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0, 0);
+  } else {
+
+    gtk_table_attach (GTK_TABLE (fields), label,
+		      0, 2, rows - 1, rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0, 0);
+    gtk_table_attach (GTK_TABLE (fields), frame,
+		      0, 2, rows - 1, rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0, 0);
+  }
+
+  submitter = new MultipleChoiceSubmitter (name, description,
+					   choices, advanced, tree_view);
   submitters.push_back (submitter);
 }
 
@@ -1058,7 +1178,8 @@ void
 FormDialog::editable_set (const std::string name,
 			  const std::string description,
 			  const std::set<std::string> values,
-			  const std::set<std::string> proposed_values)
+			  const std::set<std::string> proposed_values,
+			  bool advanced)
 {
   GtkWidget *label = NULL;
   GtkWidget *scroll = NULL;
@@ -1077,8 +1198,7 @@ FormDialog::editable_set (const std::string name,
 
   EditableSetSubmitter *submitter = NULL;
 
-  rows++;
-  gtk_table_resize (GTK_TABLE (fields), rows, 2);
+  grow_fields (advanced);
 
   /* The label */
   label = gtk_label_new (NULL);
@@ -1087,12 +1207,6 @@ FormDialog::editable_set (const std::string name,
   gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), label_text);
   g_free (label_text);
 
-  gtk_table_attach (GTK_TABLE (fields), label,
-                    0, 2, rows - 1, rows,
-                    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-                    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-                    0, 0);
-
   /* The GtkListStore containing the values */
   list_store = gtk_list_store_new (EditableSetSubmitter::COLUMN_NUMBER,
 				   G_TYPE_BOOLEAN, G_TYPE_STRING);
@@ -1151,13 +1265,31 @@ FormDialog::editable_set (const std::string name,
     }
   }
 
-  rows++;
-  gtk_table_resize (GTK_TABLE (fields), rows, 2);
-  gtk_table_attach (GTK_TABLE (fields), frame,
-                    0, 2, rows - 1, rows,
-                    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-                    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-                    0, 0);
+  if (advanced) {
+
+    gtk_table_attach (GTK_TABLE (advanced_fields), label,
+		      0, 2, advanced_rows - 1, advanced_rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0, 0);
+    gtk_table_attach (GTK_TABLE (advanced_fields), frame,
+		      0, 2, advanced_rows - 1, advanced_rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0, 0);
+  } else {
+
+    gtk_table_attach (GTK_TABLE (fields), label,
+		      0, 2, rows - 1, rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0, 0);
+    gtk_table_attach (GTK_TABLE (fields), frame,
+		      0, 2, rows - 1, rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0, 0);
+  }
 
   hbox = gtk_hbox_new (FALSE, 2);
   entry = gtk_entry_new ();
@@ -1173,15 +1305,26 @@ FormDialog::editable_set (const std::string name,
 		    (GCallback) editable_set_add_value_clicked_cb,
 		    (gpointer) entry);
 
-  rows++;
-  gtk_table_resize (GTK_TABLE (fields), rows, 2);
-  gtk_table_attach (GTK_TABLE (fields), hbox,
-		    0, 2, rows - 1, rows,
-		    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-		    (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
-		    0, 0);
 
-  submitter = new EditableSetSubmitter (name, description, tree_view);
+  grow_fields (advanced);
+
+  if (advanced) {
+
+    gtk_table_attach (GTK_TABLE (advanced_fields), hbox,
+		      0, 2, advanced_rows - 1, advanced_rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0, 0);
+  } else {
+
+    gtk_table_attach (GTK_TABLE (fields), hbox,
+		      0, 2, rows - 1, rows,
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+		      0, 0);
+  }
+
+  submitter = new EditableSetSubmitter (name, description, advanced, tree_view);
   submitters.push_back (submitter);
 }
 
@@ -1206,3 +1349,17 @@ FormDialog::cancel ()
   gtk_widget_hide_all (GTK_WIDGET (window));
   request.cancel ();
 }
+
+void
+FormDialog::grow_fields (bool advanced)
+{
+  if (advanced) {
+
+    advanced_rows++;
+    gtk_table_resize (GTK_TABLE (advanced_fields), advanced_rows, 2);
+  } else {
+
+    rows++;
+    gtk_table_resize (GTK_TABLE (fields), rows, 2);
+  }
+}
