@@ -125,7 +125,6 @@ XWindow::XWindow()
    snprintf (_colorFormat, sizeof(_colorFormat), "NONE");
   _planes = 0;
   _colorConverter = NULL;
-  _frameBuffer = NULL;
 
 #ifdef HAVE_SHM
   _XShmInfo.shmaddr = NULL;
@@ -175,9 +174,6 @@ XWindow::~XWindow()
 
   if (_colorConverter)
     delete (_colorConverter);
-
-  if (_frameBuffer)
-    free (_frameBuffer);
 }
 
 
@@ -200,8 +196,9 @@ XWindow::Init (Display* dp,
   PTRACE(4, "X11\tInitiasing new X11 window with " << windowWidth << "x" << windowHeight << " at " << x << "," << y);
   XLockDisplay (_display);
 
-  if (PTrace::CanTrace (4)) 
-    DumpVisuals();
+#if PTRACING
+  DumpVisuals();
+#endif
 
   if (!CreateAtomsAndWindow(gc, x, y, windowWidth, windowHeight)) {
     XUnlockDisplay(_display);
@@ -273,7 +270,7 @@ XWindow::Init (Display* dp,
   if (!_colorConverter)
     return 0;
 
-  _frameBuffer = (uint8_t*) malloc (_imageWidth * _imageHeight * _planes);
+  _frameBuffer = std::tr1::shared_ptr<void> (malloc (_imageWidth * _imageHeight * _planes), free);
 
   // detect the window manager type
   _wmType = GetWMType ();
@@ -302,7 +299,7 @@ XWindow::PutFrame (uint8_t* frame,
   if ((_state.curWidth != _XImage->width) || (_state.curHeight!=_XImage->height))
     CreateXImage(_state.curWidth, _state.curHeight);
 
-  _colorConverter->Convert((BYTE*)frame, (BYTE*)_frameBuffer);
+  _colorConverter->Convert((BYTE*)frame, (BYTE*)_frameBuffer.get ());
 
   pixops_scale ((guchar*) _XImage->data,
                  0,0,
@@ -311,7 +308,7 @@ XWindow::PutFrame (uint8_t* frame,
                  _planes,                   //dest_channels,
                  FALSE,                     //dest_has_alpha,
 
-                 (const guchar*) _frameBuffer,
+		(const guchar*) _frameBuffer.get (),
                  width,
                  height,
                  width * _planes,           //src_rowstride
@@ -1162,10 +1159,10 @@ XWindow::DumpVisuals()
     if (visuals != NULL) {
         for (i = 0; i < nbVisuals; i++) {
             PTRACE(4, "X11\tVisual #"  << i << " ID: " << visuals[i].visualid
-                       << " Class: "   << visuals[i].c_class  
+                       << " Class: "   << visuals[i].c_class
                        << " BPRGB: "     << visuals[i].bits_per_rgb
                        << " Depth: "   << visuals[i].depth << std::hex
-                       << " Red: 0x"   << visuals[i].red_mask 
+                       << " Red: 0x"   << visuals[i].red_mask
                        << " Green: 0x" << visuals[i].green_mask
                        << " Blue 0x"   << visuals[i].blue_mask << std::dec);
         }
